/*
	SambaScanner - A tool to scan Samba networks for files.
	Copyright (C) 2005-2007 Johannes Bauer
	
	This file is part of SambaScanner.

    SambaScanner is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; this program is ONLY licensed under
	version 2 of the License, later versions are explicitly excluded.

    SambaScanner is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SambaScanner; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include <stdio.h>
#include <pthread.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <errno.h>

#include "Semaphore.h"
#include "Mutex.h"

static struct Semaphore_Data Exec_Semaphore;
static struct Semaphore_Data Hostcount_Semaphore;
static pthread_mutex_t Memory_Lock;
static pthread_mutex_t Signal_Lock;
static pthread_mutex_t Processes_Lock;
int Signal_Count = 0;

void Init_Semaphores(int MaxHostCount) {
	Semaphore_Init(&Exec_Semaphore, 1);
	Semaphore_Init(&Hostcount_Semaphore, MaxHostCount);
	pthread_mutex_init(&Memory_Lock, NULL);
	pthread_mutex_init(&Signal_Lock, NULL);
	pthread_mutex_init(&Processes_Lock, NULL);
}

void Lock_Exec() {
	P(&Exec_Semaphore);
}

void Unlock_Exec() {
	V(&Exec_Semaphore);
}

void ThreadCnt_Inc() {
	P(&Hostcount_Semaphore);
}

void ThreadCnt_Dec() {
	V(&Hostcount_Semaphore);
}

void ThreadCnt_Set(int MaxHostCount) {
	Semaphore_Set(&Hostcount_Semaphore, MaxHostCount);
}

void BlockSignals() {
	sigset_t Mask;
	pthread_mutex_lock(&Signal_Lock);
	Signal_Count++;
	printf("Lock: %d\n", Signal_Count);
	if (Signal_Count == 1) {
		sigemptyset(&Mask);
		sigaddset(&Mask, SIGCHLD);
		if (pthread_sigmask(SIG_BLOCK, &Mask, NULL)) {
			printf("Error: %s\n", strerror(errno));
		}
		printf("BLOCKED!!!\n");
	}
	pthread_mutex_unlock(&Signal_Lock);
}

void UnblockSignals() {
	sigset_t Mask;
	pthread_mutex_lock(&Signal_Lock);
	Signal_Count--;
	printf("Unlock: %d\n", Signal_Count);
	if (Signal_Count == 0) {
		sigemptyset(&Mask);
		sigaddset(&Mask, SIGCHLD);
		if (pthread_sigmask(SIG_UNBLOCK, &Mask, NULL)) {
			printf("Error\n");
		}
		printf("UNBLOCKED!!!\n");
	}
	pthread_mutex_unlock(&Signal_Lock);
}

int pthread_create_nosig(pthread_t *tid, void *(*start_routine)(void*), void *arg) {
	int Result;
	BlockSignals();
	Result = pthread_create(tid, NULL, start_routine, arg);
	UnblockSignals();
	return Result;
}

void Lock_Processes() {
	pthread_mutex_lock(&Processes_Lock);
}

void Unlock_Processes() {
	pthread_mutex_unlock(&Processes_Lock);
}

